/*  batchprocessingwizard_v192.js: Batch processing script for BrainVoyager 21.4 and higher

    Release candidate
 
    This batch processing bundle consists of batchprocessingwizard.js and batchprocessingwizard.ui, place all in
    /Documents/BVQXExtensions/Scripts/ folder (for Mac OS X: also Plugins_64 is possible).
    Functionality: Create *.fmr/*.dmr and/or preprocess multiple *.fmr files until creation of mesh time course (*.mtc) files.
    Usage: In the BrainVoyager menu, select 'Scripts' > 'batchprocessingwizard v1.x'.
    Note: This version is for BrainVoyager QX 3.2 or higher, since PreprocessdFileName now includes the path.
    Latest update: 
 
    31-10-19:
        - (v1.9.2) replace 'CreateProject...()' by 'CreateDocument...()'
    09-01-18:
        - fix 'Cancel' button for file selection (did not work)
    04-10-17:
        - *.ui: set decimals from 2 to 5 decimals in VTC preprocessing - spatial smoothing spinbox - like in the BrainVoyager GUI
        - *.ui: for spatial filtering of FMR, set 'Gaussian smoothing' to default
    02-10-17: 
        - *.ui: changed label for temporal low-pass filtering from 'TR' to 'data points'
    28-09-17:
        - more care when splitting the name from index in text file via ':' (for Windows)
    26-09-17:
        - *.ui: from 2 to 4 decimals for VTC preprocessing - temporal smoothing spinbox
    17-09-17:
        - *.ui: make maximum TR 99999 ms instead of 9999
    12-09-17:
        - adapted writeProjectNamesToTextFileAllStages() to current text file format
    11-09-17:
        - removed the space boxes on the *.ui file for VTC and MTC
        - moved extended cerebellum box to the first tab of the *.ui file
        - added check for VDW in MNI space in CreateVTCsOrVDWs() (it is not possible to create VDWs in MNI space at the moment)
        - if FMR and VTC creation are checked, then coregistration is compulsory
    07-09-17:
        - Enabled the checkbox for the MOCO-movie
        - The FMR for ISA is not asked the first time, because 'success' was initially set to false; fixed now.
        - The Cancel button did not work, arranged now with reject() function in QtCreator
    06-09-17:
        - added function selectDMRnameAndSaveInProjFileNames() and changed file selection around line 394
        - around line 374: ISA-FMR was only asked under certain conditions; now will always be asked when the ISA box is checked
    04-08-17:
        - changed format of batch text file, added extra information per filename, separated by ':'
    02-08-17:
        - added check for selected space
    10-04-17: 
        - added getTargetSpace() and changed the dropdownbox on the main tab to space indicator, to be used to select the space-specific VMR; changed the reference to radiobuttons in createVTCsOrVDWs() to this dropdownbox.
    30-03-17:
        - added space selection radiobuttons for preprocessing VTC
        - getTargetSpaceVTC() (later: changed into getTargetSpace())
    28-03-17:
        - added getFiles()
    20-03-17:
        - added chbRemove for removing intermediate preprocessing files and flag 'preprocessedBefore'
    14-03-17:
        - remove targetfilenames in MotionCorrection
        - change indices to projfilenames[]
        - startImageRegistration: create names of IA/FA
        - adapt VTC creation function
    06-03-17:
         - added spbUseAttachedAMR on *.ui/*.js
         - reordered slots for projfilenames: fixed indices for filenames of each filetype, even if not all steps are used; if not, the filename is set to "none"
         - selectFMRnameAndSaveInProjFileNames()
         - selectVMRnameAndSaveInProjFileNames()
         - selectVTCnameAndSaveInProjFileNames()
         - selectSRFnameAndSaveInProjFileNames()
         - readTextFileIntoProjectNamesArrayAllStages()
         - writeProjectNamesToTextFileAllStages()
    05-03-17/06-03-17:
        - getFilesManuallyAllStages()
    02-03-17/06-03-17:
        - add code for registration - div + startImageRegistration()
        - partly updated functional spec (incl. updating flowchart)
    27-02-17 (v1.9.0):
        - GUI: merged 'native space' and 'normalised space' options to 'volumetric data processing'
        - GUI: added image registration tab
        - GUI: added MNI option for VTC creation
        - replaced 'BrainVoyagerQX' by 'BrainVoyager'
    19-08-16 (v1.8.3): line 532: added condition for reading text file with normalised file names: if not creating files, then only preprocessing
    01-07-16 (v1.8.3): added start button variable code; fixed bug in line 474 and 283, which caused filtering of VTCs not to take place in case first VTCs were created, because reference to array used 1d instead of 2d index
    03-07-15 (v1.8.2): resolution for *.uff files is read from *.uff name (in getUFFFile())
    29-06-15 (v1.8.1): set "mosaic" size to 96x96 (version unavailable)
    26-06-15 (v1.8): added option to select *.uff - getUFFFile() and modifications to other functions
    21-04-15 (v1.7): added option to rename filenames (chbRename, rbtnManual, rbtnFilename, btnLoadTextfile);
    March 15 (v1.7): add editing fields for filenames
    29-01-15 (v1.6): set number of decimals for dspbFWHMTGS to 5.
    29-08-14 (v1.6):
         - line 1506: fix naming of lpSizeBox (dspbFWHMTGS instead of dspbFWHMTHP)
         - fixed name for when raw file has no extension (line 1110)
    26-08-14 (v1.6): replaced all calls to plugin by script functions
    15-07-14 (v1.6):
        line 160: replace call to getFileName() by BrowseFile() including file number
    30-06-2014 (v1.6):
        - prints intermediate selected number of files to BrainVoyager QX log tab
        - added mean intensity adjustment as preprocessing step for FMR projects
        - added slice scan time correction for multiband EPI
        - 3 instead of 2 decimals for manual setting of the pixel size in FMR/DMR Project creation
        - files without extension (2d_seq for Bruker) are now be accepted
    13-03-2014 (v1.5): added support for user-defined format (*.uff), changed order preprocessing (first spatial smoothing, then temporal filtering; line 616 and further)
    10-07-2012 (v1.4): added option for determination of VTC bounding box size via coordinates.
    09-07-2012 (v1.3c): adapted to new feature in QX 2.4.2 that UseBoundingBoxForVTCCreation needs to be set. This is set to false in this version.
    09-07-2012 (v1.3b): fixed bug that all MTCs get the same name
    14-03-2012 (v1.3): added temporal high-pass filter "GLM approach" 
    21-12-2011 (v1.2): corrected small bug for creating VTC files in readTextFileIntoProjectNamesArray()
    22-11-2011 (v1.2): added VTC/VDW/MTC creation and VTC preprocessing
    08-02-2011 (v1.1): changed path for creation of preprocessed files, added more printing of the preprocessing parameters to the log tab.
    12-10-2010: added intra-session alignment    

        ~ help functions
  x      getNativeProcessingFiles
  x      writeToTextFile
  x      readTextFileIntoProjectNamesArray
 
        getPath
   x     getNormalisedProcessingFiles
   x     getSurfaceProcessingFiles
        >>
        getVTCVDWVMRName
        getMTCVMRName
        getFileName
        [noErrors] obsolete
        getProjectTypeString

        ~ `core' functions
        startProcessing
        startSliceScanTimeCorrection
        startMotionCorrection
        startTemporalFiltering
        createProjects
        createVTCsOrVDWs
        startPreprocessingVTCs
        createMTCs

    Format of the batch file:
 
     1: *.dcm/*.REC, etc
     2: *.fmr/*.dmr
     3: *.fmr for MOCO-ISA
     4: *.vmr for coreg
     5: *_IA.trf
     6: *_FA.trf
     7: *_ACPC.trf/*MNI.trf
     8: *_tal.trf
     9: *.vtc
     10: *.srf
     11: *.vmr for ACPC/TAL/MNI space
     12: *.mtc
 
    Hester Breman, Brain Innovation, 2007-2019.
*/

var scriptObj = new Object; 
scriptObj.scriptNameForUser = "batchprocessingwizard"; 
scriptObj.dialogFileName = "batchprocessingwizard_v192.ui";//"batchprocessingwizard_v19.ui";
scriptObj.dialogAccessName = "batchDlg"; 
BrainVoyager.PrintToLog("batchprocessingwizard (v1.9.2) started...");
//BrainVoyager.PrintToLog("*** Please note: this version is still being tested ***");
BrainVoyager.PrintToLog("BrainVoyager version: " + BrainVoyager.VersionMajor + "." + BrainVoyager.VersionMinor);

var $ = BrainVoyager.FindChild; // function handle
var projfilenames;// = new Array();
var projfilenamessize = 0;
//var targetfilenames = new Array(); // 1.9: now obsolete
//var targetfilenamessize = 0;
var cancel = false;
//const FORBVQX22 = true;
const PROJECTTYPE_UNKNOWN       = 0;
const PROJECTTYPE_FMR           = 1;
const PROJECTTYPE_FMR_MOSAIC    = 2;
const PROJECTTYPE_FMR_LOOPING   = 3;
const PROJECTTYPE_DMR           = 4;
const PROJECTTYPE_DMR_MOSAIC    = 5;
const UNKNOWN_SPACE             = 0;
const NATIVE_SPACE              = 1;
const AC_PC_SPACE               = 2;
const TALAIRACH_SPACE           = 3;
const MNI_SPACE                 = 4;
const VOLUMETRIC_SPACE          = 5;
//const NORMALISED_SPACE        = 2;
const SURFACE_SPACE             = 6;
const INTERPOLATION_UNKNOWN     = 0;
const INTERPOLATION_NN          = 1;
const INTERPOLATION_TRILINEAR   = 2;
const INTERPOLATION_SINC        = 3;
const TEXTFILE_FOR_PROJECTCREATION      = 1;
const TEXTFILE_FOR_NORMALISATION        = 2;
const TEXTFILE_FOR_VTCPREPROCESSING     = 3;
const TEXTFILE_FOR_SURFACEPROCESSING    = 4;
const FILTER_THP_NONE           = 0;
const FILTER_THP_LINEAR         = 1;
const FILTER_THP_FFT            = 2;
const FILTER_THP_GLMFOURIER     = 3;
const FILTER_THP_GLMDCT         = 4;
const STAGE_UNKNOWN             = 0;
const STAGE_CREATE_VTC          = 1;
const STAGE_PREPROCESS_VTC      = 2;
const STAGE_CREATE_MTC          = 3;

// global widgets
var startButton, cancelButton;
// widgets on global tab
var cmbSelectProcSpace, createProjectsBox, chbMIA, doMotionCorrBox, doSliceTimeCorrBox, doSpatialFilterBox, doTempFilterBox, chbRename, chbRemove;
var mocoOnlyBox, mocoAndISABox;
var thpAllBox, thpLinearBox, thpNoneBox, tlpNoBox, tlpYesBox, sfNoBox, sfYesBox;
var chbFMRVMRCoreg, chbCreateVTCVDWs, chbVTCSpatialSmoothing, chbVTCTemporalFiltering;
var rbtnVTCNoTHP, rbtnVTCLinearHP, rbtnVTCAllTHP, rbtnVTCNoTLP, rbtnVTCTLP;
var chbCreateMTCs;
// widgets on create FMR/DMR projects tab
var createAMRbox, filetypeBox, bigendianBox, selectSrcFilesBox, nrSlicesBox, nrVolsBox, skipNrVolsBox, xMosResBox, yMosResBox, isMosaicBox, nrMosaicVols, imgXResBox, imgYResBox, isDMRbox, isFMRbox, stcPrefixBox, sliceloopsBox, isFloatBox, isIntBox, bytesPerPixelBox; 
var istCheckBox, istValueBox, trCheckBox, trValueBox, xCheckBox, xValueBox, yCheckBox, yValueBox, zCheckBox, zValueBox;
var rbtnFileFormatManuf, rbtnFileFormatUser, btnLoadUFF;
// widgets on FMR preprocessing parameters tab
var mocoLessDataBox, mocoLogBox, mocoInterpBox, mocoNrIterBox, mocoTargetVolBox, mocoISABox, chbMovie;
var scsInterpBox, scsAscBox, scsAscIntBox, scsAscInt2Box, scsDescBox, scsDescIntbox, scsDescInt2Box;
var sgsMMbox, sgsPxBox, sgsSizeBox, rbtnGLMFourier, rbtnGLMDCT, rbtnFFT, spbFourierCycles, thpCycBox, thpHzBox, thpSizeBox, lpSecBox, lpTRBox, lpSizeBox;
// widgets on image registration tab
var rbtnFMRVMRCoreg, spbUseAttachedAMR, rbtnFMRVMRCoregBBR;
// widgets on create VTC/VDW tab
var rbtnSelectVTC, rbtnSelectVDW, rbtnVTCVDWdtypeFloat, rbtnVTCVDWdtypeInt, rbtnSelectNativeSpace, rbtnSelectACPCSpace, rbtnSelectTalSpace, rbtnSelectMNISpace;
var chbExtendCerebellum, rbtnResolution1mm3, rbtnResolution2mm3, rbtnResolution3mm3, cmbVTCVDWInterpolation, spbIntensityThreshold;
var rbtnCoordBBox, rbtnIntensityThresh, rbtnDefaultBBox, spbXfrom, spbXto, spbYfrom, spbYto, spbZfrom, spbZto;
// widgets on preprocess VTC tab
var rbtnVTCPreprNativeSpace, rbtnVTCPreprACPCSpace, rbtnVTCPreprTalSpace, rbtnVTCPreprMNISpace;
var rbtnVTCmm, rbtnVTCvox, dspbFWHMSpatialSmooth, rbtnVTCHz, rbtnVTCCyc;
var dspbVTCFWHMTHP, rbtnVTCSec, rbtnVTCDataPoints, dspbVTCFWHMTGS;
// widgets on create MTC tab
var rbtnMTCNativeSpace, rbtnMTCACPCSpace, rbtnMTCTalSpace, rbtnMTCMNISpace;
var dspbMTCFrom, dspbMTCTo;
// widgets on files tab
var nroffilesBox, getFileButton, selectFilesNowBox, selectTextFileBox, lblFilesSelected;
//var selectFilesNowBox2, selectTextFileBox2, btnGetVTCVDWnames, btnVTCVDWGetVMRName, lblVTCVDWVMRname, spbNrVTCVDWs;
//var selectFilesNowBox3, selectTextFileBox3, btnGetMTCnames, btnMTCGetVMRName, lblMTCVMRname, spbNrMTCs;
// widgets on Rename tab
var leFile1, leFile2, leFile3, leFile4, leFile5, leFile6, leFile7, leFile8, leFile9, leFile10, leFile11, leFile12, leFile13, leFile14, leFile15, leFile16,
leFile17, leFile18, leFile19, leFile20;
// global variables
var preprocessedBefore = false;

// ~~~~~~~~~~~~~~~~~~~~~~~ File I/O ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


scriptObj.writeProjectNamesToTextFileAllStages = function(projfilenames, stagedescr, otherdir) {
    
    var filecounter, projfilecounter;
    var success;
    var prefixes = new Array("1 scanner: ", "2 fmr: ", "3 fmr ISA: ", "4 vmr for coreg: ", "5 IA.trf: ", "6 FA.trf: ", "7 ACPC.trf: ", "8 tal: ", "9 vtc: ", "10 srf: ", "11 vmr target space: ", "12 mtc: ");
    
    // write the textfile to the path of the first file
    var name;
    if (otherdir) {
        name = String(this.getHigherPath(projfilenames[0][0]) + "filenames_for_batchprocessingwizard_stage_" + stagedescr + "_v19.txt"); //BV stops due to file in dcmdir
    } else {
        name = String(this.getPath(projfilenames[0][0]) + "filenames_for_batchprocessingwizard_stage_" + stagedescr + "_v19.txt");
    }
    BrainVoyager.PrintToLog("Start writing file: " + name);
    var filenamesfile = new QFile(name);
    filenamesfile.open(new QIODevice.OpenMode(QIODevice.WriteOnly));
    var textstr = new QTextStream(filenamesfile);
    textstr.writeDouble(projfilenames.length);
    textstr.writeString("\n\n");
    for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
        for (projfilecounter = 0; projfilecounter < 12; projfilecounter++) {
            textstr.writeString(prefixes[projfilecounter] + projfilenames[filecounter][projfilecounter] + "\n");
        }
        textstr.writeString("\n");
    }
    filenamesfile.close();
    BrainVoyager.PrintToLog("Wrote " + name + ".");
    return success;
}

scriptObj.readTextFileIntoProjectNamesArrayAllStages = function() {

    var filecounter;
    var success = true;
    var textfilename = BrainVoyager.BrowseFile("Please select the text file with filenames", "*.txt");
    BrainVoyager.PrintToLog("Start reading: " + textfilename);
    var name = String(textfilename); // circumvention of problem with implicit strings
    var filenamesfile = new QFile(name);
    filenamesfile.open(new QIODevice.OpenMode(QIODevice.ReadOnly));
    var textstr = new QTextStream(filenamesfile);
    var nroffiles = parseInt(textstr.readLine());
    BrainVoyager.PrintToLog("Number of projects: " + nroffiles);
    projfilenames = new Array(nroffiles);// projfilenames has not yet been used in the process
    var dummy = textstr.readLine(); // empty line after number of projects
    
 /*   var iswin = false;
    var sys = new QSysInfo();
    var os = sys.productType();
    if (os == "windows") {
        iswin = true;
        BrainVoyager.PrintToLog("Running on Windows.");
    } else {
        BrainVoyager.PrintToLog("Not running on Windows.");
    }*/

    for (filecounter = 0; filecounter < nroffiles; filecounter++) {
        
        var filesPerProject = new Array(11);
        var projfilecounter;
        for (projfilecounter = 0; projfilecounter < 11; projfilecounter++) {
            dummy = textstr.readLine();
            var pos = dummy.indexOf(":");
            var line = dummy.substring(pos+1, dummy.length);
           // var line=dummy.split(':'); // => does not work on Windows with : in filename
            //project.vdmFile0 = textstr.readLine(); //vdm file:
            //      var filename = line[1].replace(/(^\s+|\s+$)/g, ""); // this regular expression removes spaces
            var filename = line.trim();
            BrainVoyager.PrintToLog("File " + (projfilecounter+1) + ":\n" + "'" + filename + "'");
            filesPerProject[projfilecounter] = filename;
        }
        var dummy = textstr.readLine(); // 04-08-17: empty line after each project
        dummy = textstr.readLine();
        projfilenames[filecounter] = filesPerProject;
    }
    filenamesfile.close();
}


scriptObj.getPath = function(filename) {

	var start = 0; // start searching from begin
	var last = filename.lastIndexOf("/");
	var path = filename.substring(start, (last+1)); // now return path without filename
	return path;
}

scriptObj.getHigherPath = function(filename) {
    
    var start = 0; // start searching from begin
    var last = filename.lastIndexOf("/");
    var tmppath = filename.substring(start, (last)); // now return path without filename
    last = filename.lastIndexOf("/");
    var tmppath = filename.substring(start, (last+1));
    return path; // hopefully one directory higher
}

scriptObj.getFiles = function() {

    if ((createProjectsBox.checked && chbCreateVTCVDWs.checked) && (!chbFMRVMRCoreg.checked)) {
        chbFMRVMRCoreg.checked = true;
        BrainVoyager.PrintToLog("Coregistration required due to combined FMR/DMR project and VTC/VDW creation.");
    }
    
    if (selectTextFileBox.checked) {
        this.readTextFileIntoProjectNamesArrayAllStages();
    } else {
        this.getFilesManuallyAllStages();
    }
}

scriptObj.getFilesManuallyAllStages = function() {

    /* projfilenames:
     
     0: *.dcm/*.REC, etc
     1: *.fmr/*.dmr
     2: *.fmr for MOCO-ISA
     3: *.vmr for coreg
     4: *_IA.trf
     5: *_FA.trf
     6: *_ACPC.trf/*MNI.trf
     7: *_tal.trf
     8: *.vtc
     9: *.srf
     10: *.vmr (*APCP/TAL/MNI.vmr)
     11: *.mtc
     */
    
    var success = true;
    var nroffiles = nroffilesBox.value;
    var targetspace = this.getTargetSpace();
    var filesPerProject;
    if (nroffilesBox.value > 0) {
        
        projfilenames = new Array(nroffiles);
        var preprocessingFMR = false;
        BrainVoyager.PrintToLog("Start selecting: " + nroffiles + " files...");
        for (filecounter = 0; filecounter < nroffiles; filecounter++) {
            
            filesPerProject = new Array("none", "none", "none", "none", "none", "none", "none", "none", "none", "none", "none", "none");
            var filetype = "*.*";
            
            // native files:    *.dcm
            if ((createProjectsBox.checked == 1) && success) {
                var rawfiletype = filetypeBox.currentText;
                if (rawfiletype == "DICOM") {
                    filetype = "*.dcm";
                } else if (rawfiletype == "SIEMENS") {
                    filetype = "*.IMA";
                } else if (rawfiletype == "PHILIPS_REC") {
                    filetype = "*.REC";
                } else if (rawfiletype == "GE_I") {
                    filetype = "*.*";
                } else if (rawfiletype == "GE_MR") {
                    filetype = "*.MR";
                } else if (rawfiletype == "ANALYZE") {
                    filetype = "*.hdr";
                } else if (rbtnFileFormatUser.checked) {
                    filetype = "*";
                } // end filetype
                
                var projfilename = BrainVoyager.BrowseFile("Please select the file for project number " + (filecounter+1), filetype);
                if (projfilename.length > 2) {
                    filesPerProject[0] = projfilename;
                    if (chbRename.checked) {
                        var currentwidget=$("leFile"+(filecounter+1));
                        currentwidget.text = filesPerProject[0];//projfilenames[filecounter][0]; // set raw img name
                    } // end chbRename
                } else {
                    success = false;
                    break;//added 09-01-17
                } // end projfilename.length
            } // end createProjectsBox.checked
            
            // preprocess FMR:  *.fmr (+ *.fmr MOCO-ISA)
            // check if any of the preprocessing boxes is checked
            preprocessingFMR = (chbMIA.checked || doSliceTimeCorrBox.checked || doMotionCorrBox.checked || doSpatialFilterBox.checked || doTempFilterBox.checked);
            if ((!createProjectsBox.checked) && preprocessingFMR && success) {
                success = this.selectFMRnameAndSaveInProjFileNames(filecounter, false, filesPerProject);
            }
            if (mocoISABox.checked && success) {
                success = this.selectFMRnameAndSaveInProjFileNames(filecounter, true, filesPerProject);
            }
            
            // coregistration:  *.fmr + *.vmr
            if (chbFMRVMRCoreg.checked && success) {
                
                // *.fmr files have not yet been created or selected
                if ((!createProjectsBox.checked) && (!preprocessingFMR)) {
                    success = this.selectFMRnameAndSaveInProjFileNames(filecounter, false, filesPerProject);
                }
                // now select *.vmr files
                success = this.selectVMRnameAndSaveInProjFileNames(filecounter, filesPerProject, true, NATIVE_SPACE);
            }
            
            // create VTC/VDW:  *.fmr + *.vmr + *IA.trf + *FA.trf (+ *ACPC.trf) (+ *tal.trf)
            if (chbCreateVTCVDWs.checked && success) {
            
                // *.fmr files have not been created or selected
                if ((!createProjectsBox.checked) && (!preprocessingFMR)) {
                    
                    if ((!rbtnSelectVDW.checked)) { //  && (!chbFMRVMRCoreg.checked)
                      success = this.selectFMRnameAndSaveInProjFileNames(filecounter, false, filesPerProject);
                    } else {
                     success = this.selectDMRnameAndSaveInProjFileNames(filecounter, filesPerProject);
                    }
                    
                }
                // now select *.vmr files
                if (!chbFMRVMRCoreg.checked) {
                    success = this.selectVMRnameAndSaveInProjFileNames(filecounter, filesPerProject, false, targetspace);
                } else {
                    if (targetspace == NATIVE_SPACE) {
                        var vmrname = filesPerProject[3];
                        filesPerProject[10] = vmrname;
                    } else {
                        success = this.selectVMRnameAndSaveInProjFileNames(filecounter, filesPerProject, false, targetspace);
                    }
                }
               // *IA.trf + *FA.trf (+ *ACPC.trf) (+ *tal.trf): have not been selected in earlier stages; will be generated in coregistration process
                if ((!chbFMRVMRCoreg.checked) && success) {
                    filename = BrainVoyager.BrowseFile("Please select the initial alignment file for project number " + (filecounter+1), "*IA.trf");
                    if (filename != "") {//(this.noErrors()) {
                        filesPerProject[4] = filename;
                    } else {
                        success = false;
                    }
                    filename = BrainVoyager.BrowseFile("Please select the fine alignment file for project number " + (filecounter+1), "*FA.trf");
                    if (filename != "") {//(this.noErrors()) {
                        filesPerProject[5] = filename;
                    } else {
                        success = false;
                    }
                }

                if (((targetspace == AC_PC_SPACE) || (targetspace == TALAIRACH_SPACE) && success)) {
                    
                    filename = BrainVoyager.BrowseFile("Please select the AC-PC alignment file for project number " + (filecounter+1), "*ACPC.trf");
                    if (filename != "") {//(this.noErrors()) {
                       // projfilenames[filecounter][6] = filename;
                        filesPerProject[6] = filename;
                    } else {
                        success = false;
                    }
                }
                
                if ((targetspace == MNI_SPACE) && success) {
                    filename = BrainVoyager.BrowseFile("Please select the MNI aligment file for project number " + (filecounter+1), "*a12.trf");
                    if (filename != "") {//(this.noErrors()) {
                       // projfilenames[filecounter][7] = filename;
                        filesPerProject[6] = filename;
                    } else {
                        success = false;
                    }
                }

                if ((targetspace == TALAIRACH_SPACE) && success) {
                    filename = BrainVoyager.BrowseFile("Please select the Talairach landmarks file for project number " + (filecounter+1), "*.tal");
                    if (filename != "") {//(this.noErrors()) {
                        filesPerProject[7] = filename;
                    } else {
                        success = false;
                        break;
                    }
                }
            } // end create VTC/VDW
 
            // preprocess VTC:  *.vmr + *.vtc
            if ((chbVTCSpatialSmoothing.checked) || (chbVTCTemporalFiltering.checked) && success) {
                
                // check if VMR is available
                if (!chbCreateVTCVDWs.checked) {
                    success = this.selectVMRnameAndSaveInProjFileNames(filecounter, filesPerProject, false, this.getTargetSpace());
                    success = this.selectVTCnameAndSaveInProjFileNames(filecounter, filesPerProject);
                }
            }
            
            // create MTC: *.vmr + *.vtc + *.srf
            if (chbCreateMTCs.checked && success) {
                if ((!chbCreateVTCVDWs.checked) && (!chbCreateVTCVDWs.checked) && (!chbVTCSpatialSmoothing.checked) && (!chbVTCTemporalFiltering.checked)) {
                    success = this.selectVMRnameAndSaveInProjFileNames(filecounter, filesPerProject, false, this.getTargetSpace());// TAL/MNI-VMR, perhaps
                }
                if (!(chbVTCSpatialSmoothing.checked || chbVTCTemporalFiltering.checked || chbCreateVTCVDWs.checked)) {
                    success = this.selectVTCnameAndSaveInProjFileNames(filecounter, filesPerProject);
                }
                success = this.selectSRFnameAndSaveInProjFileNames(filecounter, filesPerProject);
            }
            if (success) {
                projfilenames[filecounter] = filesPerProject;
            }
            
        } // end filecounter
        
        this.writeProjectNamesToTextFileAllStages(projfilenames, "input", true);
            
    } else {
        BrainVoyager.writeLog("Number of files to select is 0.");
    }
        
}

scriptObj.getTargetSpace = function() {

    var targetspace;
    var selection = cmbSelectProcSpace.currentText;//currentIndex; 02-08-17: currentIndex does not seem to work anymore (?)
    
    if (selection == "Native space") {
        targetspace = NATIVE_SPACE;
    } else if (selection == "AC-PC space") {
        targetspace = AC_PC_SPACE;
    } else if (selection == "Talairach space") {
        targetspace = TALAIRACH_SPACE;
    } else if (selection == "MNI space") {
         targetspace = MNI_SPACE;
    } else {
        targetspace = UNKNOWN_SPACE;
    }
    return targetspace;
}


scriptObj.selectFMRnameAndSaveInProjFileNames = function(filecounter, isForISA, filesPerProject) {
    
    var success = false;
    var filename = BrainVoyager.BrowseFile("Please select the FMR for project number " + (filecounter+1), "*.fmr");
    if (filename != "") {//(this.noErrors()) {
        if (isForISA) {
            filesPerProject[2] = filename;
        } else {
            filesPerProject[1] = filename;
        }
        success = true;
    } else {
        success = false;
    }
    return success;
}

scriptObj.selectDMRnameAndSaveInProjFileNames = function(filecounter, filesPerProject) {
    
    var success = false;
    var filename = BrainVoyager.BrowseFile("Please select the DMR for project number " + (filecounter+1), "*.dmr");
    if (filename != "") {//(this.noErrors()) {
        filesPerProject[1] = filename;
        success = true;
    } else {
        success = false;
    }
    return success;
}

scriptObj.selectVMRnameAndSaveInProjFileNames = function(filecounter, filesPerProject, isForCoregistration, targetspace) {

    var success = false;
    var filename;
        if (isForCoregistration) {
            filename = BrainVoyager.BrowseFile("Please select the VMR in native space for project number " + (filecounter+1), "*.vmr"); // should be in native space
            filesPerProject[3] = filename;
        } else {
            if (targetspace == AC_PC_SPACE) {
                filename = BrainVoyager.BrowseFile("Please select the VMR for project number " + (filecounter+1), "*ACPC.vmr");
            } else if (targetspace == TALAIRACH_SPACE) {
                filename = BrainVoyager.BrowseFile("Please select the VMR for project number " + (filecounter+1), "*TAL.vmr");
            } else if (targetspace == MNI_SPACE) {
                filename = BrainVoyager.BrowseFile("Please select the VMR for project number " + (filecounter+1), "*MNI.vmr");
            } else {
                filename = BrainVoyager.BrowseFile("Please select the VMR for project number " + (filecounter+1), "*.vmr");
            }
            filesPerProject[10] = filename;
        }
    if (filename.length > 2) {
        success = true;
    }

    return success;
}

scriptObj.selectVTCnameAndSaveInProjFileNames = function(filecounter, filesPerProject) {
    
    var success = false;
    var filename = BrainVoyager.BrowseFile("Please select the VTC for project number " + (filecounter+1), "*.vtc");
    if (filename != "") {//(this.noErrors()) {
        filesPerProject[8] = filename;
        success = true;
    } else {
        success = false;
    }
    return success;
}

scriptObj.selectSRFnameAndSaveInProjFileNames = function(filecounter, filesPerProject) {
 
    var success = true;
    var filename = BrainVoyager.BrowseFile("Please select the SRF for project number " + (filecounter+1), "*.srf");
    if (filename != "") {//(this.noErrors()) {
        filesPerProject[9] = filename;
    } else {
        success = false;
    }
    return success;
}


scriptObj.getVTCVDWVMRName = function() {

    var vmrname = BrainVoyager.BrowseFile("Please select the anatomical file", "*.vmr");//this.getFileName("*.vmr");
    lblVTCVDWVMRname.text = vmrname;
}
 
  
scriptObj.getMTCVMRName = function() {

    var vmrname = BrainVoyager.BrowseFile("Please select the anatomical file", "*.vmr");//this.getFileName("*.vmr");
    lblMTCVMRname.text = vmrname; 
}

scriptObj.getUFFFile = function() {

    var name = BrainVoyager.BrowseFile("Please select the user-defined file", "*.uff");
    BrainVoyager.PrintToLog("Selected: " + name);
    
    var pospath = name.lastIndexOf("/");
    var posext = name.lastIndexOf(".");
    var uffBasicName = name.substring((pospath+1), posext);
    filetypeBox.insertItem(0, uffBasicName);
    filetypeBox.setCurrentIndex(0);
    
    // infer the resolution from the namevar re=/\d*\w+\d*/g;
    var re=/\d*_\d*/g;
    var result = String(re.exec(uffBasicName));
    BrainVoyager.PrintToLog("Result: " + result);
    var arr = result.split("_");
    var validResolution = false;
    
    var minres = 10;
    var maxres = 10000;
    var len = arr.length;
    var resArr = new Array();
    for (i=0;i<len;i++) {
        if (!isNaN(arr[i])) {
            if (arr[i] > minres && arr[i] < maxres) {
                resArr.push(arr[i]);
            }
        }
    }
    
    if (resArr.length == 2) {
        xMosResBox.value = resArr[0];
        yMosResBox.value = resArr[1];
        imgXResBox.value = resArr[0];
        imgYResBox.value = resArr[1];
        BrainVoyager.PrintToLog("Set the following image matrix: " + resArr[0] + " x " + resArr[1]);
    } else {
        BrainVoyager.PrintToLog("Did not find matrix size in *.uff file name.");
    }
}


scriptObj.getFileName = function(filetype) {

    var filename = BrainVoyager.BrowseFile("Please select the file", filetype);
    return filename;
}
    
// ~~~~~~~~~~~~~~~~~~~~~~~ help functions ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

/*scriptObj.noErrors = function() {

    var noerror;
    var errornr = BrainVoyager.GetPluginIntParameter("ErrorNr");
    if (errornr != -1) {
        noerror = true;
    } else {
        var errormsg = BrainVoyager.GetPluginStringParameter("ErrorMsg");
        BrainVoyager.PrintToLog(errormsg);
        noerror = false;        
    }
    return noerror;
}   */

scriptObj.getProjectTypeString = function(projecttype) {

    switch (projecttype) {
    
        case PROJECTTYPE_FMR:           return "FMR";
        case PROJECTTYPE_FMR_MOSAIC:    return "FMR mosaic";
        case PROJECTTYPE_FMR_LOOPING:   return "FMR slices x time";
        case PROJECTTYPE_DMR:           return "DMR";
        case PROJECTTYPE_DMR_MOSAIC:    return "DMR mosaic";
        default:                        return "Onbekend";

    }
}
    
    

// ~~~~~~~~~~~~~~~~~~~~~~~ main processing function ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.startProcessing = function() {
    
    var success;
    BrainVoyager.PrintToLog("Start processing...");
    
    if (projfilenames.length == 0) {
        BrainVoyager.PrintToLog("No input files have been selected. Process cancelled.");
    } else if (this.getTargetSpace == UNKNOWN_SPACE){
        BrainVoyager.PrintToLog("No space been selected. Process cancelled.");
    } else {
    
            if (createProjectsBox.checked == 1) {
                this.createProjects();
            }
            if (chbMIA.checked == 1) {
                this.startMeanIntensityAdjustment();
            }
            if (doSliceTimeCorrBox.checked == 1) {
                this.startSliceScanTimeCorrection();
            }
            if (doMotionCorrBox.checked == 1) {
                this.startMotionCorrection();
            }
            if (doSpatialFilterBox.checked == 1) {
                this.startSpatialFiltering();
            }
            if (doTempFilterBox.checked == 1) {
                this.startTemporalFiltering();
            }
            if (chbFMRVMRCoreg.checked == 1) {
                success = this.startImageRegistration();
            }
            if (chbCreateVTCVDWs.checked) {
                success = this.createVTCsOrVDWs(); // take into account if files were created in chbFMRVMRCoreg.checked()
            }
            if (chbVTCSpatialSmoothing.checked || chbVTCTemporalFiltering.checked) {
                this.startPreprocessingVTCs();
            }
            if (chbCreateMTCs.checked) {
                
                success = this.createMTCs();
            }
    } // projfilenames.length > 0
    //writeProjectNamesToTextFileAllStages();
    this.batchDlg.close();
} 

 
// ~~~~~~~~~~~~~~~~~~~~~~~ FMR Preprocessing ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
scriptObj.startMeanIntensityAdjustment = function() {
        
        var success = true;
        
        if (!cancel) {
            var filecounter, bvqxproject, resultfilename;
            for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
                BrainVoyager.PrintToLog("Open " + projfilenames[filecounter][1]);//projfilenames[filecounter]);
                bvqxproject = BrainVoyager.OpenDocument(projfilenames[filecounter][1]);
                success = bvqxproject.AdjustMeanIntensity();
                if (success) {
                    resultfilename = bvqxproject.FileNameOfPreprocessdFMR;
                    BrainVoyager.PrintToLog("New file name: " + resultfilename);
                    projfilenames[filecounter][1] = resultfilename; // place new name back in old name array
                } else {
                    BrainVoyager.PrintToLog("Mean intensity adjustment for this project did not succeed.");
                    success = false;
                }
                bvqxproject.Close();
                preprocessedBefore = true;
            }
            BrainVoyager.PrintToLog("Finished mean intensity adjustment.");
            this.writeProjectNamesToTextFileAllStages(projfilenames, "meanintensityadjustment", false);
        } else {
            BrainVoyager.PrintToLog("Procedure was cancelled.");
        }
}
    
scriptObj.startSliceScanTimeCorrection = function() {

	BrainVoyager.PrintToLog("Start slice scan time correction...");
    var interpolationmethod = scsInterpBox.currentIndex;  
    if (interpolationmethod == 3) {
        cancel = true;
        BrainVoyager.PrintToLog("No interpolation method selected for slice scan time correction.");
    } else {
        BrainVoyager.PrintToLog("Interpolation method parameter: " + interpolationmethod); 
    }

    var sliceorder;
    if (scsAscBox.checked == 1) {
        sliceorder = 0;
    } else if (scsAscIntBox.checked == 1) {
        sliceorder = 1;
    } else if (scsAscInt2Box.checked == 1) {
        sliceorder = 2;
    } else if (scsDescBox.checked == 1) {
        sliceorder = 10;
    } else if (scsDescIntbox.checked == 1) {
        sliceorder = 11;
    } else if (scsDescInt2Box.checked == 1) {
        sliceorder = 12;
    }
    BrainVoyager.PrintToLog("Slice order parameter: " + sliceorder);
    
    if (!cancel) {
        var filecounter;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            BrainVoyager.PrintToLog("Open " + projfilenames[filecounter][1]);
            var bvqxproject = BrainVoyager.OpenDocument(projfilenames[filecounter][1]);
            if (bvqxproject.HasSliceTimeTable) { // new BVQX 2.8.2
                BrainVoyager.PrintToLog("Slice scan time correction for multiband EPI...");
                bvqxproject.CorrectSliceTimingUsingTimeTable(1); // 1: cubic spline interpolation
            } else {
                bvqxproject.CorrectSliceTiming(sliceorder, interpolationmethod);
            }
            var resultfilename = bvqxproject.FileNameOfPreprocessdFMR;
            BrainVoyager.PrintToLog("New file name: " + resultfilename);
            projfilenames[filecounter][1] = resultfilename; // place new name back in old name array
            if (chbRemove.checked && preprocessedBefore) {
                bvqxproject.Remove(); // only delete previous project when one has chosen to delete
            } else {
                bvqxproject.Close();
            }
            preprocessedBefore = true;
        }
        BrainVoyager.PrintToLog("Finished slice scan time correction.");
        this.writeProjectNamesToTextFileAllStages(projfilenames, "slicescantimecorrection", false);
    } else {
		BrainVoyager.PrintToLog("Procedure was cancelled.");
	}
}

scriptObj.startMotionCorrection = function() { 
    
    var success = true;
    // check parameters
    var targetVol = mocoTargetVolBox.value;
    var interpolationOpt = mocoInterpBox.currentIndex;
    if (interpolationOpt < 0) cancel = true;
    var reducedData = mocoLessDataBox.checked;
    var nrIterations = mocoNrIterBox.value;
    var movies = chbMovie.checked;
    var writeLog = mocoLogBox.checked;    
    var doISA = mocoISABox.checked;
    BrainVoyager.PrintToLog("Selected options for motion correction: ");
    BrainVoyager.PrintToLog("Target volume: " + targetVol);
    BrainVoyager.PrintToLog("Interpolation method: " + mocoInterpBox.currentIndex + ", which should be " + mocoInterpBox.currentText);   
    BrainVoyager.PrintToLog("Reduced data: " + reducedData);
    BrainVoyager.PrintToLog("Number of iterations: " + nrIterations);
    BrainVoyager.PrintToLog("Movies: " + (movies ? "yes" : "no"));
    BrainVoyager.PrintToLog("Write extended log file: " + writeLog);
    BrainVoyager.PrintToLog("Perform intra-session alignment: " + doISA);

    if (!cancel) {
        var filecounter, bvqxproject, resultfilename;
        if (doISA) {
            
            // perform motion correction with intra-session alignment
                 for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
                    BrainVoyager.PrintToLog("Open " + projfilenames[filecounter][1]); // get *.fmrs from index [][1]
                    bvqxproject = BrainVoyager.OpenDocument(projfilenames[filecounter][1]);
                      success = bvqxproject.CorrectMotionTargetVolumeInOtherRunEx(projfilenames[filecounter][2], targetVol, interpolationOpt, (!reducedData), nrIterations, movies, writeLog);
                    if (success) {
                        resultfilename = bvqxproject.FileNameOfPreprocessdFMR;
                        BrainVoyager.PrintToLog("New file name: " + resultfilename);
                        projfilenames[filecounter][1] = resultfilename; // place new name back in old name array
                    } else {
                        BrainVoyager.PrintToLog("Motion correction and intra-session alignment of this project did not succeed.");
                    }
                    if (chbRemove.checked && preprocessedBefore) {
                         bvqxproject.Remove(); // only delete when current file is a preprocessed project and one has chosen to delete intermediate files
                     } else {
                         bvqxproject.Close();
                     }
                     preprocessedBefore = true;
                }
        } else {
            for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
                BrainVoyager.PrintToLog("Open " + projfilenames[filecounter]);
                bvqxproject = BrainVoyager.OpenDocument(projfilenames[filecounter][1]);
                success = bvqxproject.CorrectMotionEx(targetVol, interpolationOpt, (!reducedData), nrIterations, movies, writeLog);
                if (success) {
                    resultfilename = bvqxproject.FileNameOfPreprocessdFMR;
                    BrainVoyager.PrintToLog("New file name: " + resultfilename);
                    projfilenames[filecounter][1] = resultfilename; // place new name back in old name array
                } else {
                    BrainVoyager.PrintToLog("Motion correction of this project did not succeed.");
                }
                
                if (chbRemove.checked && preprocessedBefore) {
                    bvqxproject.Remove(); // only delete file when current file is mean intensity or slice scan time corrected and one has chosen to delete
                } else {
                    bvqxproject.Close();
                }
                preprocessedBefore = true;
            }
        }
        BrainVoyager.PrintToLog((doISA ? "Finished motion correction and intra-session alignment" : "Finished motion correction."));
            this.writeProjectNamesToTextFileAllStages(projfilenames, (doISA ? "isa" : "motioncorrection"), false);
    } else {
		BrainVoyager.PrintToLog("Procedure was cancelled: no valid interpolation option.");
	}
}

scriptObj.startTemporalFiltering = function() { 

    const FILTER_THP_NONE           = 0;
    const FILTER_THP_LINEAR         = 1;
    const FILTER_THP_FFT            = 2;
    const FILTER_THP_GLMFOURIER     = 3;
    const FILTER_THP_GLMDCT         = 4;

    var success = true;
    var thptype, thpBasisFuncs, thpUnit, thpValue, lpUnit, lpValue;
    var allTHP = thpAllBox.checked; 
    BrainVoyager.PrintToLog("Selected options for temporal high-pass filtering: ");
    if (thpNoneBox.checked) {
        thptype = FILTER_THP_NONE;
        BrainVoyager.PrintToLog("None");
    } else if (thpLinearBox.checked) {
        thptype = FILTER_THP_LINEAR; 
    } else {
        if (rbtnGLMFourier.checked) {
            thptype = FILTER_THP_GLMFOURIER;
            thpBasisFuncs = spbFourierCycles.value;
            BrainVoyager.PrintToLog("GLM Fourier with cycles: " + thpBasisFuncs);
        } else if (rbtnGLMDCT.checked) {
            thptype = FILTER_THP_GLMDCT;
            thpBasisFuncs = spbFourierCycles.value;
            BrainVoyager.PrintToLog("GLM Discrete Cosine Transform with basis functions: " + thpBasisFuncs);
        } else if (rbtnFFT.checked) {
            thptype = FILTER_THP_FFT;
            if (thpCycBox.checked) {
                thpUnit = "cycles";
            } else {
                thpUnit = "Hz";
            }
            var thpValue = thpSizeBox.value;
            BrainVoyager.PrintToLog("Unit: " + thpUnit);
            BrainVoyager.PrintToLog("Value: " + thpValue);            
        }
    }
    
    var noTLP = tlpNoBox.checked; 
    var doTLP = tlpYesBox.checked;
    if (lpSecBox.checked) {
        lpUnit = "s";
    } else {
        lpUnit = "TR";
    }
    lpValue = lpSizeBox.value;
    if (doTLP) {
        BrainVoyager.PrintToLog("Selected options for temporal low-pass filtering: ");
        BrainVoyager.PrintToLog("Unit: " + lpUnit);
        BrainVoyager.PrintToLog("Value: " + lpValue);
    }

    if (!cancel) {
        var filecounter, bvqxproject, resultfilename;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            BrainVoyager.PrintToLog("Open " + projfilenames[filecounter][1]);
            
            // high pass temporal filtering
            if (thptype > FILTER_THP_NONE) {
                // hack
                var name = new String(projfilenames[filecounter][1]);
                bvqxproject = BrainVoyager.OpenDocument(name.toString());
                if (thptype == FILTER_THP_GLMFOURIER) {
                    success = bvqxproject.TemporalHighPassFilterGLMFourier(thpBasisFuncs);
                } else if (thptype == FILTER_THP_GLMDCT) {
                    success = bvqxproject.TemporalHighPassFilterGLMDCT(thpBasisFuncs);
                } else if (thptype == FILTER_THP_FFT) {
                    success = bvqxproject.TemporalHighPassFilter(thpValue, thpUnit); 
                } else if (thptype == FILTER_THP_LINEAR) {
                    success = bvqxproject.LinearTrendRemoval();
                }
                if (success) {
                    resultfilename = bvqxproject.FileNameOfPreprocessdFMR;
                    BrainVoyager.PrintToLog("New file name: " + resultfilename);
                    projfilenames[filecounter][1] = resultfilename; // place new name back in old name array
                } else {
                    BrainVoyager.PrintToLog("Temporal high-pass filtering of this project did not succeed.");
                }
                if (chbRemove.checked && preprocessedBefore) {
                    bvqxproject.Remove(); // only delete file when current file is preprocessed and one has chosen to delete intermediate files
                } else {
                    bvqxproject.Close();
                }
                preprocessedBefore = true;
            }
            // low pass temporal filtering
            if (doTLP) {
                bvqxproject = BrainVoyager.OpenDocument(projfilenames[filecounter][1]);
                success = bvqxproject.TemporalGaussianSmoothing(lpValue, lpUnit);
                if (success) {
                    //resultfilename = this.getPath(projfilenames[filecounter]) + bvqxproject.FileNameOfPreprocessdFMR;
                    resultfilename = bvqxproject.FileNameOfPreprocessdFMR;
                    BrainVoyager.PrintToLog("New file name: " + resultfilename);
                    projfilenames[filecounter][1] = resultfilename; // place new name back in old name array
                } else {
                    BrainVoyager.PrintToLog("Temporal low-pass filtering of this project did not succeed.");
                }
                if (chbRemove.checked && preprocessedBefore) {
                    bvqxproject.Remove(); // only delete file when current file is preprocessed and one has chosen to delete intermediate files
                } else {
                    bvqxproject.Close();
                }
                preprocessedBefore = true;
            }
        }
        BrainVoyager.PrintToLog("Finished temporal filtering.");
        this.writeProjectNamesToTextFileAllStages(projfilenames, "temporalfiltering", false);
    } else {
		BrainVoyager.PrintToLog("Procedure was cancelled.");
	}
 
}

scriptObj.startSpatialFiltering = function() { 

    var success = true;
    var sgsValue = sgsSizeBox.value;
    var sgsUnit;
    if (sgsMMbox.checked) {
        sgsUnit = "mm";
    } else {
        sgsUnit = "px";
    }
    BrainVoyager.PrintToLog("Selected options for spatial low-pass filtering (gaussian smoothing): ");
    BrainVoyager.PrintToLog("Unit: " + sgsUnit);
    BrainVoyager.PrintToLog("Value: " + sgsValue);

    if (!cancel) {
        var filecounter, bvqxproject, resultfilename;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            BrainVoyager.PrintToLog("Open " + projfilenames[filecounter][1]);
            bvqxproject = BrainVoyager.OpenDocument(projfilenames[filecounter][1]);
            success = bvqxproject.SpatialGaussianSmoothing(sgsValue, sgsUnit);
            if (success) {
                resultfilename = bvqxproject.FileNameOfPreprocessdFMR;
                BrainVoyager.PrintToLog("New file name: " + resultfilename);
                projfilenames[filecounter][1] = resultfilename;  // since bv1.9
            } else {
                BrainVoyager.PrintToLog("Spatial filtering for this project did not succeed.");
            }
            if (chbRemove.checked && preprocessedBefore) {
                bvqxproject.Remove(); // only delete file when current file is preprocessed and one has chosen to delete intermediate files
            } else {
                bvqxproject.Close();
            }
            preprocessedBefore = true;
        }
        BrainVoyager.PrintToLog("Finished spatial filtering.");
        this.writeProjectNamesToTextFileAllStages(projfilenames, "spatialfiltering", false);
    } else {
		BrainVoyager.PrintToLog("Procedure was cancelled.");
	}
}
	
// ~~~~~~~~~~~~~~~~~~~~~~~ Create FMR/DMRs ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
scriptObj.createProjects = function() {
    
	// properties of ProjectFactory class
    var currentprojecttype;

    // declare and initialize member properties
	var filetype; 
	var firstfilename;
	var nrofvols; 
	var nrofvolsskip; 
	var createamr; 
	var nrslices; 
	var stcprefix; 
	var swapbytes; 	
	var mosaicx; 
	var mosaicy; 
	var nrbytes; 
	var savingdir;
	var volsinimg; 
	var imgresx; 
	var imgresy; 
    
		BrainVoyager.PrintToLog("Retrieve create project parameters...");
    
        // parameters for determination of project type
        var isdmr = isDMRbox.checked;
        var ismosaic = isMosaicBox.checked;
        var isfmr = isFMRbox.checked;
        var looping = sliceloopsBox.checked;
        if (isdmr && ismosaic) {
            this.currentprojecttype = PROJECTTYPE_DMR_MOSAIC;
        } else if (isdmr) {
            this.currentprojecttype = PROJECTTYPE_DMR;    
        } else if (isfmr && looping) {
            this.currentprojecttype = PROJECTTYPE_FMR_LOOPING;
        } else if (isfmr && ismosaic) {
            this.currentprojecttype = PROJECTTYPE_FMR_MOSAIC;  
        } else if (isfmr) {
            this.currentprojecttype = PROJECTTYPE_FMR;
        }
        BrainVoyager.PrintToLog("Project type: " + this.getProjectTypeString(this.currentprojecttype));
    
        // project-specific parameters    
    BrainVoyager.PrintToLog("Project parameters:");
        this.filetype = filetypeBox.currentText;
    BrainVoyager.PrintToLog("First file name: " + this.firstfilename);
        this.nrofvols = nrVolsBox.value;
    BrainVoyager.PrintToLog("Number of volumes: " + this.nrofvols);
        this.nrofvolsskip = skipNrVolsBox.value;
    BrainVoyager.PrintToLog("Number of skipped volumes: " + this.nrofvolsskip);
        this.createamr = createAMRbox.checked;
    BrainVoyager.PrintToLog("Create pseudo AMR: " + this.createamr);
        this.nrslices = nrSlicesBox.value;
    BrainVoyager.PrintToLog("Number of slices: " + this.nrslices);
        this.stcprefix = stcPrefixBox.text;
    BrainVoyager.PrintToLog("STC prefix: " + this.stcprefix);
        this.swapbytes = bigendianBox.checked;
    BrainVoyager.PrintToLog("Swap bytes: " + this.swapbytes);
        this.mosaicx = xMosResBox.value; 
    BrainVoyager.PrintToLog("Size of (mosaic) image (x-axis): " + this.mosaicx);
    this.mosaicy = yMosResBox.value;
    BrainVoyager.PrintToLog("Size of (mosaic) image (y-axis): " + this.mosaicy);
    this.nrbytes = bytesPerPixelBox.value;
    BrainVoyager.PrintToLog("Number of bytes: " + this.nrbytes);
    this.volsinimg = (nrMosaicVols.currentIndex+1);
    BrainVoyager.PrintToLog("Number of volumes in mosaic file: " + this.volsinimg);
    this.imgresx = imgXResBox.value;
    BrainVoyager.PrintToLog("Size of individual image (x-axis): " + this.imgresx);
    this.imgresy = imgYResBox.value;
    BrainVoyager.PrintToLog("Size of individual image (y-axis): " + this.imgresy);
    
        var project;
        var filecounter, bvqxproject, newname;
    
    if (chbRename.checked) {
        if (projfilenames.length > 20) {
            BrainVoyager.PrintToLog("Number of files too large for renaming via user interface, sorry.");
            chbRename.checked = false; // TEST! new 02-08-17
        } else {
            for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
                projfilenames[filecounter][1] = 'leFile'+(filecounter+1).text;
            }
        }
    }
    
    for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            
            BrainVoyager.PrintToLog("Start creating document " + (filecounter+1) + " from:\n" + projfilenames[filecounter][0]);
            this.firstfilename = projfilenames[filecounter][0];
            this.savingdir = this.getPath(this.firstfilename);
        
            switch (this.currentprojecttype) {
  
                case PROJECTTYPE_FMR:
                    project = BrainVoyager.CreateDocumentFMR(this.filetype, this.firstfilename, this.nrofvols, this.nrofvolsskip, this.createamr, this.nrslices,
                        this.stcprefix, this.swapbytes, this.imgresx, this.imgresy, this.nrbytes, this.savingdir);
                    break;
                case PROJECTTYPE_FMR_MOSAIC:
                    project = BrainVoyager.CreateDocumentMosaicFMR(this.filetype, this.firstfilename, this.nrofvols, this.nrofvolsskip, this.createamr, this.nrslices,
                        this.stcprefix, this.swapbytes, this.mosaicx, this.mosaicy, this.nrbytes, this.savingdir, this.volsinimg, this.imgresx, this.imgresy);
                    break;
                case PROJECTTYPE_FMR_LOOPING:
                    project = BrainVoyager.CreateDocumentFMRSlicesTimeLooping(this.filetype, this.firstfilename, this.nrofvols, this.nrofvolsskip, this.createamr,
                    this.nrslices, this.stcprefix, this.swapbytes, this.mosaicx, this.mosaicy, this.nrbytes, this.savingdir, this.volsinimg, this.imgresx, this.imgresy);
                    break;  
                case PROJECTTYPE_DMR:
                    project = BrainVoyager.CreateDocumentDMR(this.filetype, this.firstfilename, this.nrofvols, this.nrofvolsskip, this.createamr, this.nrslices,
                        this.stcprefix, this.swapbytes, this.imgresx, this.imgresy, this.nrbytes, this.savingdir);
                    break;    
                case PROJECTTYPE_DMR_MOSAIC:    
                    project = BrainVoyager.CreateDocumentMosaicDMR(this.filetype, this.firstfilename, this.nrofvols, this.nrofvolsskip, this.createamr, this.nrslices,
                        this.stcprefix, this.swapbytes, this.mosaicx, this.mosaicy, this.nrbytes, this.savingdir, this.volsinimg, this.imgresx, this.imgresy);
                    break;
                default:
                    BrainVoyager.PrintToLog("Projecttype unknown.");
            }          

            if (istCheckBox.checked == 1) {
                BrainVoyager.PrintToLog("Current InterSliceTime value: " + project.InterSliceTime);
                project.InterSliceTime = istValueBox.value;
                BrainVoyager.PrintToLog("New InterSliceTime value: " + project.InterSliceTime);
            } 
            if (trCheckBox.checked == 1) {
                BrainVoyager.PrintToLog("Current TR value: " + project.TR);
                project.TR = trValueBox.value;
                project.TimeResolutionVerified = true;
                BrainVoyager.PrintToLog("New TR value: " + project.TR);
            }
            if (xCheckBox.checked == 1) {
                BrainVoyager.PrintToLog("Current pixel x-resolution: " + project.PixelSizeOfSliceDimX);
                project.PixelSizeOfSliceDimX = xValueBox.value;
                BrainVoyager.PrintToLog("New pixel x-resolution: " + project.PixelSizeOfSliceDimX);
                project.VoxelResolutionVerified = true;
            }
            if (yCheckBox.checked == 1) {
                BrainVoyager.PrintToLog("Current pixel y-resolution: " + project.PixelSizeOfSliceDimY);
                project.PixelSizeOfSliceDimY = yValueBox.value;
                BrainVoyager.PrintToLog("New pixel x-resolution: " + project.PixelSizeOfSliceDimY);
                project.VoxelResolutionVerified = true;
            }
            if (zCheckBox.checked == 1) {
                BrainVoyager.PrintToLog("Current slice thickness: " + project.SliceThickness);
                project.SliceThickness = zValueBox.value; 
                BrainVoyager.PrintToLog("New slice thickness: " + project.SliceThickness);
                project.VoxelResolutionVerified = true;   
            }

                 
            if (chbRename.checked) {
                var currentwidget=$("leFile"+(filecounter+1));
                newname = currentwidget.text;
            } else if ((this.firstfilename).indexOf(".") < 0) { // filename without extension
                newname = this.firstfilename + (isfmr ? ".fmr" : ".dmr");
            } else {
                newname = this.firstfilename.substring(0, (this.firstfilename.length-3)) + (isfmr ? "fmr" : "dmr");
            }
            BrainVoyager.PrintToLog("New name: " + newname);
            project.SaveAs(newname);
            projfilenames[filecounter][1] = newname; // since v1.9
            project.Close();
    } // end filecounter
    this.writeProjectNamesToTextFileAllStages(projfilenames, "projectcreation", false);
}

// ~~~~~~~~~~~~~~~~~~~~~~~ FMR-VMR Coregistration ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.startImageRegistration = function() {

    BrainVoyager.PrintToLog("Start image registration...");

    // start processing
    var project;
    var filecounter, bvqxproject, newname;
    
    for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
        
        var vmr = BrainVoyager.OpenDocument(projfilenames[filecounter][3]);
        if (rbtnFMRVMRCoregBBR.checked == 1) {
            success = vmr.CoregisterFMRToVMRUsingBBR(projfilenames[filecounter][1]); // parameter: fmr name
        } else {
            success = vmr.CoregisterFMRToVMR(projfilenames[filecounter][1], spbUseAttachedAMR.value); // parameter: fmr name, use AMR: 0=no 1=pseudo AMR 2=real AMR (extra hi-res scan)
        }
        
        // create IA and FA filename
        var epiName = projfilenames[filecounter][1];
        var epiFile = new QFile(projfilenames[filecounter][1]); // FMR
        var epiFileInfo = new QFileInfo(epiFile);
        var anatFile = new QFile(projfilenames[filecounter][3]); // VMR
        var anatFileInfo = new QFileInfo(anatFile);
        var anatimg = projfilenames[filecounter][3];
        var loc, base;
        if (anatimg.indexOf("\\") < 0) { // Mac, Linux
            loc = anatimg.indexOf("/");
            base = anatimg.substring(loc, anatimg.length);
        } else {
            loc = anatimg.indexOf("\\"); // Windows
            base = anatimg.substring(loc, anatimg.length);
        }
        loc = epiName.lastIndexOf(".");
        base = epiName.substring(0, epiName.length-4); // 2nd parameter specifies length of substring, not location.
        
        var ianame = base + '-TO-' + anatFileInfo.baseName() + '_IA.trf';
        var faname = base + '-TO-' + anatFileInfo.baseName() + '_FA.trf';
        BrainVoyager.PrintToLog("Inferred *_IA.trf name: " + ianame);
        BrainVoyager.PrintToLog("Inferred *_FA.trf name: " + faname);
        projfilenames[filecounter][4] = ianame;
        projfilenames[filecounter][5] = faname;
        vmr.Close();
    }
}


// ~~~~~~~~~~~~~~~~~~~~~~~ Create VTC/VDWs ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.createVTCsOrVDWs = function() {

    BrainVoyager.PrintToLog("Start creating VTC(s) or VDW(s)...");

    // get parameters
    var vmr;
    var doCreateVTC = rbtnSelectVTC.checked;
    var dataType = (rbtnVTCVDWdtypeFloat.checked ? 2 : 1); // 1: int16, 2: float32
    var resolution;
    if (rbtnResolution1mm3.checked) {
        resolution = 1;
    } else if (rbtnResolution2mm3.checked) {
        resolution = 2;
    } else {
        resolution = 3;
    }
    
    var interpolation           = (cmbVTCVDWInterpolation.currentIndex - 1); // 0: nearest neighbour, 1: trilinear, 2: sinc
    var useExtendedBoundingBox  = chbExtendCerebellum.checked;
    var vmrname                 = projfilenames[0][10];//             = lblVTCVDWVMRname.text; ////>>>>>>>>>>>>>>>>
    
    // determine bounding box size method
    var useDefaultBBx           = rbtnDefaultBBox.checked;
    var useIntensityThresh      = rbtnIntensityThresh.checked;
    var useBBxCoords            = rbtnCoordBBox.checked;
    // just get all bounding box size values
    var threshold                = spbIntensityThreshold.value;
    var xStart                   = spbXfrom.value;
    var xEnd                     = spbXto.value;
    var yStart                   = spbYfrom.value;
    var yEnd                     = spbYto.value;
    var zStart                   = spbZfrom.value;
    var zEnd                     = spbZto.value;

        // start processing
        var project;
        var filecounter, bvqxproject, newname, vmrname;
        BrainVoyager.PrintToLog("VTC/VDW creation parameters:\n" + "interpolation: " + interpolation + "\nthreshold: " + threshold);
        BrainVoyager.PrintToLog("Use extended bounding box: " + (useExtendedBoundingBox ? "yes" : "no")); 
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            
            BrainVoyager.PrintToLog("Start creating VTC/VDW " + (filecounter+1) + " from:\n" + projfilenames[filecounter][1]);
            vmrname = projfilenames[filecounter][10];
            var nameSrc     = projfilenames[filecounter][1];
            var nameIAfile  = projfilenames[filecounter][4];
            var nameFAfile  = projfilenames[filecounter][5];
            var targetspace = this.getTargetSpace();
            vmr = BrainVoyager.OpenDocument(vmrname);     // VMR in target space
            // preparation for creating VTC name
            this.firstfilename = nameSrc;
            this.savingdir = this.getPath(this.firstfilename);
            
            if (useBBxCoords) {
                vmr.UseBoundingBoxForVTCCreation    = true; // values will be adjusted to fit on multiple of resolution
                vmr.TargetVTCBoundingBoxXStart      = xStart; // values not changed (here X/Y) use default (TAL) bounding box values
                vmr.TargetVTCBoundingBoxXEnd        = xEnd;
                vmr.TargetVTCBoundingBoxYStart      = yStart;
                vmr.TargetVTCBoundingBoxYEnd        = yEnd;
                vmr.TargetVTCBoundingBoxZStart      = zStart; 
                vmr.TargetVTCBoundingBoxZEnd        = zEnd;
            } else {
                vmr.UseBoundingBoxForVTCCreation = false;
            }

            if (targetspace == NATIVE_SPACE) {

                // create VTC/VDW name
                newname = this.firstfilename.substring(0, (this.firstfilename.length-4)) + ((doCreateVTC) ? "_NATIVE.vtc" : "_NATIVE.vdw");
                // create VTC/VDW in native space
                vmr.ExtendedTALSpaceForVTCCreation = false; // non applicable for native space
                if (doCreateVTC) {
                    success = vmr.CreateVTCInVMRSpace(nameSrc, nameIAfile, nameFAfile, newname, dataType, resolution, interpolation, threshold);
                } else {
                    success = vmr.CreateVDWInVMRSpace(nameSrc, nameIAfile, nameFAfile, newname, dataType, resolution, interpolation, threshold);
                }

            } else if (targetspace == AC_PC_SPACE) {

                // create VTC/VDW name
                newname = this.firstfilename.substring(0, (this.firstfilename.length-4)) + ((doCreateVTC) ? "_ACPC.vtc" : "_ACPC.vdw");
                // create VTC in AC-PC space
                var nameACPCfile  = projfilenames[filecounter][6];//projfilenames[filecounter][3];
                vmr.ExtendedTALSpaceForVTCCreation = false; // non applicable for AC-PC space
                if (doCreateVTC) {
                    success = vmr.CreateVTCInACPCSpace(nameSrc, nameIAfile, nameFAfile, nameACPCfile, newname, dataType, resolution, interpolation, threshold);
                } else {
                    success = vmr.CreateVDWInACPCSpace(nameSrc, nameIAfile, nameFAfile, nameACPCfile, newname, dataType, resolution, interpolation, threshold);                
                }
                
              } else if (targetspace == TALAIRACH_SPACE) {

                // create VTC/VDW name
                newname = this.firstfilename.substring(0, (this.firstfilename.length-4)) + ((doCreateVTC) ? "_TAL.vtc" : "_TAL.vdw"); 

                // create VTC in Talairach space
                var nameACPCfile  = projfilenames[filecounter][6];//projfilenames[filecounter][3];
                var nameTALfile  = projfilenames[filecounter][7];//projfilenames[filecounter][4];
                vmr.ExtendedTALSpaceForVTCCreation = useExtendedBoundingBox; // this is true or false
                if (doCreateVTC) {
                    success = vmr.CreateVTCInTALSpace(nameSrc, nameIAfile, nameFAfile, nameACPCfile, nameTALfile, newname, dataType, resolution, interpolation, threshold);
                } else {
                    success = vmr.CreateVDWInTALSpace(nameSrc, nameIAfile, nameFAfile, nameACPCfile, nameTALfile, newname, dataType, resolution, interpolation, threshold);
                }

              } else if (targetspace == MNI_SPACE) {
                
                  if (!doCreateVTC) {
                      BrainVoyager.PrintToLog("Please note that creating VDWs is not possible in MNI space via scripting.");
                      success = false;
                  } else {
                      // create VTC name
                      newname = this.firstfilename.substring(0, this.firstfilename.length-4)  + "_MNI.vtc";
                      // create VTC in MNI space
                      var nameMNIfile  = projfilenames[filecounter][6];//projfilenames[filecounter][3];
                      vmr.ExtendedTALSpaceForVTCCreation = false; // non applicable for MNI space
                      success = vmr.CreateVTCInMNISpace(nameSrc, nameIAfile, nameFAfile, nameMNIfile, newname, dataType, resolution, interpolation, threshold);
                  }
            }
            vmr.Close();
            projfilenames[filecounter][8] = newname;
            if (success) {
                BrainVoyager.PrintToLog("Created: " + newname);                
            } else {
                BrainVoyager.PrintToLog("VDW creation did not succeed.");
            }
        } // end loop
        this.writeProjectNamesToTextFileAllStages(projfilenames, (doCreateVTC ? "resulting_vtc" : "resulting_vdw"));
    return success;
}


// ~~~~~~~~~~~~~~~~~~~~~~~ Preprocess VTCs ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.startPreprocessingVTCs = function() {
  
    BrainVoyager.PrintToLog("Get preprocessing parameters...");
    // get parameters
    var SGSunit; // ("mm" or "vx")
    if (rbtnVTCmm.checked) {
        SGSunit = "mm";
    } else {
        SGSunit = "vx";
    }

    var SGSquantity = dspbFWHMSpatialSmooth.value; 
    var HPFunit; //  ("cycles" or "Hz")
    if (rbtnVTCHz.checked) {
        HPFunit = "Hz";
    } else {
        HPFunit = "cycles";
    }

    var HPFquantity = dspbVTCFWHMTHP.value; 
    var LPFunit; // ("d" or "dps" (data points) or "s" or "secs" (seconds))
    if (rbtnVTCSec.checked) {
        LPFunit = "secs";
    } else {
        LPFunit = "dps";
    }
    var LPFquantity = dspbVTCFWHMTGS.value;
      
    // start processing
    var currentvtc;
    var success;
    var filecounter;
    var vmrname = projfilenames[0][10];//lblVTCVDWVMRname.text;///>>>>>>>
    if (vmrname.length < 1) {
        BrainVoyager.PrintToLog("No VMR name selected. Preprocessing VTCs not possible.");
        success = false;
    } else {
        var vmr;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            
            vmr = BrainVoyager.OpenDocument(vmrname);
            currentvtc = projfilenames[filecounter][8];
            
            if (currentvtc != "none") {
                
                vmr.LinkVTC(currentvtc);
                BrainVoyager.PrintToLog("Current VTC file: " + vmr.FileNameOfCurrentVTC);  // show name of current VTC

                // spatial low-pass filter
                if (chbVTCSpatialSmoothing.checked) {
                    vmr.SpatialGaussianSmoothing(SGSquantity, SGSunit);    // FWHM value and unit ("mm" or "vx")
                    BrainVoyager.PrintToLog("Name of spatially smoothed VTC file: " + vmr.FileNameOfCurrentVTC);
                    projfilenames[filecounter][8] = vmr.FileNameOfCurrentVTC;
                }
                if (chbVTCTemporalFiltering.checked) {
                    
                    // temporal high-pass filter
                    if (rbtnVTCAllTHP.checked) { // high-pass temporal filter (see below) includes LTR
                        vmr.TemporalHighPassFilter(HPFquantity, HPFunit);    // HP alue and unit ("cycles" or "Hz")
                        BrainVoyager.PrintToLog("Name of VTC file without linear trends: " + vmr.FileNameOfCurrentVTC);
                        projfilenames[filecounter][8] = vmr.FileNameOfCurrentVTC;
                        
                    } else if (rbtnVTCLinearHP.checked) {
                        vmr.LinearTrendRemoval();    // FWHM value and unit ("mm" or "vx")
                        BrainVoyager.PrintToLog("Name of VTC file without linear trends: " + vmr.FileNameOfCurrentVTC);
                        projfilenames[filecounter][8] = vmr.FileNameOfCurrentVTC;
                    }
                    
                    // temporal low-pass filter
                    if (rbtnVTCTLP.checked) {
                        vmr.TemporalGaussianSmoothing(LPFquantity, LPFunit);    // FWHM value and unit ("d" or "dps" (data points) or "s" or "secs" (seconds))
                        BrainVoyager.PrintToLog("Name of temporally smoothed VTC file: " + vmr.FileNameOfCurrentVTC);
                        projfilenames[filecounter][8] = vmr.FileNameOfCurrentVTC;
                    }
                }
    
            } else {
                BrainVoyager.PrintToLog("No VTC to preprocess available.");
            }
            vmr.Close();
        } // end loop
        this.writeProjectNamesToTextFileAllStages(projfilenames, "preprocessedvtcs");
        
    } // end if success
}


// ~~~~~~~~~~~~~~~~~~~~~~~ Create MTCs ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

// the resulting textfile of the preprocessed VTCs can be used, but add the surface files following the line of each VTC filename
scriptObj.createMTCs = function() {

    BrainVoyager.PrintToLog("Create MTC(s)...");
    
    // get parameters
    var samplingdepth = dspbMTCFrom.value;
    var samplingouter = dspbMTCTo.value; 
    BrainVoyager.PrintToLog("Sampling from: " + samplingdepth.toString() + " to: " + samplingouter.toString()); 
    
    // start processing
    var currentvtc, vmr, newname, currentsrf;
    var success;
    var filecounter;
    var vmrname = projfilenames[0][10];//lblMTCVMRname.text;
    if (vmrname.length < 1) {
        success = false;
        BrainVoyager.PrintToLog("No VMR or SRF selected. Cannot create MTCs.");
        
    } else if (projfilenames.length < 1) {
        BrainVoyager.PrintToLog("No VTC filenames available. Cannot create MTCs.");
    } else {
        
            this.firstfilename = projfilenames[0][8]; // there is always a VTC to create an MTC
            this.savingdir = this.getPath(this.firstfilename);
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            vmr = BrainVoyager.OpenDocument(vmrname);
            currentvtc = projfilenames[filecounter][8];
            vmr.LinkVTC(currentvtc);
            srfname = projfilenames[filecounter][9];
            vmr.LoadMesh(srfname);
            newname = currentvtc.substring(0, (currentvtc.length-4)) + ".mtc"; // new per 04-07-12
            BrainVoyager.PrintToLog("Creating MTC from " + currentvtc);
            vmr.CreateMTCFromVTC(samplingdepth, samplingouter, newname);
            projfilenames[filecounter][11] = newname; // *.mtc
            vmr.Close();
        }
        this.writeProjectNamesToTextFileAllStages(projfilenames, "mtcfiles");
    }
    return success;
}



// ~~~~~~~~~~~~~~~~~~~~~~~ main ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.initDlg = function() { 
    
    this.batchDlg.windowTitle = "batchprocessingwizard";
    
    // ~ global widgets
    cmbSelectProcSpace      = BrainVoyager.FindChild("cmbSelectProcSpace");
    startButton             = BrainVoyager.FindChild("btnStart");// 01-07-16
    cancelButton            = BrainVoyager.FindChild("btnCancel"); // 07-09-17
    
    // volumetric space
    createProjectsBox       = BrainVoyager.FindChild("chbCreateProjects");
    chbRename               = BrainVoyager.FindChild("chbRename");
    chbRemove               = BrainVoyager.FindChild("chbRemove"); // 20-03-17
    chbMIA                  = BrainVoyager.FindChild("chbMIA");
    doSliceTimeCorrBox      = BrainVoyager.FindChild("chbSCS");
    doMotionCorrBox         = BrainVoyager.FindChild("chbMOCO");
    doSpatialFilterBox      = BrainVoyager.FindChild("chbSF");
    doTempFilterBox         = BrainVoyager.FindChild("chbTF");
    mocoOnlyBox             = BrainVoyager.FindChild("rbtnMOCO");
    mocoAndISABox           = BrainVoyager.FindChild("rbtnMOCOISA");
    thpAllBox               = BrainVoyager.FindChild("rbtnAllTHP");
    thpLinearBox            = BrainVoyager.FindChild("rbtnLinearHP");
    thpNoneBox              = BrainVoyager.FindChild("rbtnNoTHP");
    tlpNoBox                = BrainVoyager.FindChild("rbtnNoTLP");
    tlpYesBox               = BrainVoyager.FindChild("rbtnTLP");
    sfNoBox                 = BrainVoyager.FindChild("rbtnSFno");
    sfYesBox                = BrainVoyager.FindChild("rbtnSFyes");
    rbtnFileFormatManuf     = BrainVoyager.FindChild("rbtnFileFormatManuf");
    rbtnFileFormatUser      = BrainVoyager.FindChild("rbtnFileFormatUser");
    btnLoadUFF              = BrainVoyager.FindChild("btnLoadUFF");
    chbFMRVMRCoreg          = BrainVoyager.FindChild("chbFMRVMRCoreg");
    chbCreateVTCVDWs        = BrainVoyager.FindChild("chbCreateVTCVDWs");
    chbExtendCerebellum     = BrainVoyager.FindChild("chbExtendCerebellum");
    chbVTCSpatialSmoothing  = BrainVoyager.FindChild("chbVTCSpatialSmoothing");
    chbVTCTemporalFiltering = BrainVoyager.FindChild("chbVTCTemporalFiltering");
    rbtnVTCNoTHP            = BrainVoyager.FindChild("rbtnVTCNoTHP"); 
    rbtnVTCLinearHP         = BrainVoyager.FindChild("rbtnVTCLinearHP");
    rbtnVTCAllTHP           = BrainVoyager.FindChild("rbtnVTCAllTHP");
    rbtnVTCNoTLP            = BrainVoyager.FindChild("rbtnVTCNoTLP"); 
    rbtnVTCTLP              = BrainVoyager.FindChild("rbtnVTCTLP");
    // surface space
    chbCreateMTCs           = BrainVoyager.FindChild("chbCreateMTCs");

    // ~ widgets on create projects tab
    createAMRbox    = BrainVoyager.FindChild("chbCreateAMR");
    filetypeBox     = BrainVoyager.FindChild("cmbFileType");
    bigendianBox    = BrainVoyager.FindChild("chbBigEndian");
    nrSlicesBox     = BrainVoyager.FindChild("spbNrSlices");
    nrVolsBox       = BrainVoyager.FindChild("spbNrOfVols");
    skipNrVolsBox   = BrainVoyager.FindChild("spbSkipNrVols");
    xMosResBox      = BrainVoyager.FindChild("spbXRes");
    yMosResBox      = BrainVoyager.FindChild("spbYRes");
    isMosaicBox     = BrainVoyager.FindChild("chbMosaic");
    nrMosaicVols    = BrainVoyager.FindChild("cmbNrMosVols");
    imgXResBox      = BrainVoyager.FindChild("spbImgXRes");
    imgYResBox      = BrainVoyager.FindChild("spbImgYRes");
    isDMRbox        = BrainVoyager.FindChild("rbtnDMR");
    isFMRbox        = BrainVoyager.FindChild("rbtnFMR");
    stcPrefixBox    = BrainVoyager.FindChild("leStcPrefix");
    sliceloopsBox   = BrainVoyager.FindChild("chbSlicesXTime");
    bytesPerPixelBox = BrainVoyager.FindChild("spbBytesPerPixel");
    
    // ~ create projects: manual settings
    istCheckBox = BrainVoyager.FindChild("chbIST"); 
    istValueBox = BrainVoyager.FindChild("spbIST");
    trCheckBox = BrainVoyager.FindChild("chbTR"); 
    trValueBox = BrainVoyager.FindChild("spbTR"); 
    xCheckBox = BrainVoyager.FindChild("chbXpix"); 
    xValueBox = BrainVoyager.FindChild("dspbPixSizeX"); 
    yCheckBox = BrainVoyager.FindChild("chbYpix"); 
    yValueBox = BrainVoyager.FindChild("dspbPixSizeY"); 
    zCheckBox = BrainVoyager.FindChild("chbZpix"); 
    zValueBox = BrainVoyager.FindChild("dspbSliceThick");
    
    // ~ widgets on preprocessing parameter tab
    mocoLessDataBox     = BrainVoyager.FindChild("chbLessData");
    mocoLogBox          = BrainVoyager.FindChild("chbLog");
    mocoInterpBox       = BrainVoyager.FindChild("cmbInterpMOCO");
    mocoNrIterBox       = BrainVoyager.FindChild("spbNumIter");
    chbMovie            = BrainVoyager.FindChild("chbMovie");
    mocoTargetVolBox    = BrainVoyager.FindChild("spbTargetVol");
    mocoISABox          = BrainVoyager.FindChild("rbtnMOCOISA");
    scsInterpBox        = BrainVoyager.FindChild("cmBoxInterpSCS");
    scsAscBox           = BrainVoyager.FindChild("rbtnAsc");
    scsAscIntBox        = BrainVoyager.FindChild("rbtnAscInt");
    scsAscInt2Box       = BrainVoyager.FindChild("rbtnAscInt2");
    scsDescBox          = BrainVoyager.FindChild("rbtnDesc");
    scsDescIntbox       = BrainVoyager.FindChild("rbtnDescInt");
    scsDescInt2Box      = BrainVoyager.FindChild("rbtnDescInt2");
    sgsMMbox            = BrainVoyager.FindChild("rbtnMm");
    sgsPxBox            = BrainVoyager.FindChild("rbtnPx");
    sgsSizeBox          = BrainVoyager.FindChild("dspbFWHMsizeSGS");
    rbtnGLMFourier      = BrainVoyager.FindChild("rbtnGLMFourier");
    rbtnGLMDCT          = BrainVoyager.FindChild("rbtnGLMDCT");
    rbtnFFT             = BrainVoyager.FindChild("rbtnFFT");
    spbFourierCycles    = BrainVoyager.FindChild("spbFourierCycles");
    thpCycBox           = BrainVoyager.FindChild("rbtnCyc");
    thpHzBox            = BrainVoyager.FindChild("rbtnHz");
    thpSizeBox          = BrainVoyager.FindChild("dspbFWHMTHP");
    lpSecBox            = BrainVoyager.FindChild("rbtnSec");
    lpTRBox             = BrainVoyager.FindChild("rbtnTR");
    lpSizeBox           = BrainVoyager.FindChild("dspbFWHMTGS"); // fixed 29-08-14
    
    
    // ~ widgets on FMR-VMR Coregistration tab
    rbtnFMRVMRCoreg     = BrainVoyager.FindChild("rbtnFMRVMRCoreg");
    spbUseAttachedAMR   = BrainVoyager.FindChild("spbUseAttachedAMR");
    rbtnFMRVMRCoregBBR  = BrainVoyager.FindChild("rbtnFMRVMRCoregBBR");
    
    
    // ~ widgets on Create VTC/VDW tab
    rbtnSelectVTC           = BrainVoyager.FindChild("rbtnSelectVTC");
    rbtnSelectVDW           = BrainVoyager.FindChild("rbtnSelectVDW");
    rbtnVTCVDWdtypeFloat    = BrainVoyager.FindChild("rbtnVTCVDWdtypeFloat");
    rbtnVTCVDWdtypeInt      = BrainVoyager.FindChild("rbtnVTCVDWdtypeInt");
    rbtnResolution1mm3      = BrainVoyager.FindChild("rbtnResolution1mm3");
    rbtnResolution2mm3      = BrainVoyager.FindChild("rbtnResolution2mm3");
    rbtnResolution3mm3      = BrainVoyager.FindChild("rbtnResolution3mm3");
    cmbVTCVDWInterpolation  = BrainVoyager.FindChild("cmbVTCVDWInterpolation");
    spbIntensityThreshold   = BrainVoyager.FindChild("spbIntensityThreshold");
    rbtnCoordBBox           = BrainVoyager.FindChild("rbtnCoordBBox");
    rbtnIntensityThresh     = BrainVoyager.FindChild("rbtnIntensityThresh");
    rbtnDefaultBBox         = BrainVoyager.FindChild("rbtnDefaultBBox");
    spbXfrom                = BrainVoyager.FindChild("spbXfrom");
    spbXto                  = BrainVoyager.FindChild("spbXto");
    spbYfrom                = BrainVoyager.FindChild("spbYfrom");
    spbYto                  = BrainVoyager.FindChild("spbYto");
    spbZfrom                = BrainVoyager.FindChild("spbZfrom");
    spbZto                  = BrainVoyager.FindChild("spbZto");
    
    // ~ widgets on Preprocess VTCs tab
    rbtnVTCmm               = BrainVoyager.FindChild("rbtnVTCmm");
    rbtnVTCvox              = BrainVoyager.FindChild("rbtnVTCvox"); 
    dspbFWHMSpatialSmooth   = BrainVoyager.FindChild("dspbFWHMSpatialSmooth"); 
    rbtnVTCHz               = BrainVoyager.FindChild("rbtnVTCHz"); 
    rbtnVTCCyc              = BrainVoyager.FindChild("rbtnVTCCyc"); 
    dspbVTCFWHMTHP          = BrainVoyager.FindChild("dspbVTCFWHMTHP"); 
    rbtnVTCSec              = BrainVoyager.FindChild("rbtnVTCSec"); 
    rbtnVTCDataPoints       = BrainVoyager.FindChild("rbtnVTCDataPoints"); 
    dspbVTCFWHMTGS          = BrainVoyager.FindChild("dspbVTCFWHMTGS");
    
    // ~ widgets on Create MTCs tab
    dspbMTCFrom             = BrainVoyager.FindChild("dspbMTCFrom");
    dspbMTCTo               = BrainVoyager.FindChild("dspbMTCTo");
    
    // ~ widgets on files tab
    // native/scanner space
    getFileButton       = BrainVoyager.FindChild("btnGetFiles");
    nroffilesBox        = BrainVoyager.FindChild("spnBoxNrOfFiles");
    selectFilesNowBox   = BrainVoyager.FindChild("rbtnFileDialog");
    selectTextFileBox   = BrainVoyager.FindChild("rbtnTextFile");

    // yes, I know.
    leFile1             = BrainVoyager.FindChild("leFile1");
    leFile2             = BrainVoyager.FindChild("leFile2");
    leFile3             = BrainVoyager.FindChild("leFile3");
    leFile4             = BrainVoyager.FindChild("leFile4");
    leFile5             = BrainVoyager.FindChild("leFile5");
    leFile6             = BrainVoyager.FindChild("leFile6");
    leFile7             = BrainVoyager.FindChild("leFile7");
    leFile8             = BrainVoyager.FindChild("leFile8");
    leFile9             = BrainVoyager.FindChild("leFile9");
    leFile10             = BrainVoyager.FindChild("leFile10");
    leFile11             = BrainVoyager.FindChild("leFile11");
    leFile12             = BrainVoyager.FindChild("leFile12");
    leFile13             = BrainVoyager.FindChild("leFile13");
    leFile14             = BrainVoyager.FindChild("leFile14");
    leFile15             = BrainVoyager.FindChild("leFile15");
    leFile16             = BrainVoyager.FindChild("leFile16");
    leFile17             = BrainVoyager.FindChild("leFile17");
    leFile18             = BrainVoyager.FindChild("leFile18");
    leFile19             = BrainVoyager.FindChild("leFile19");
    leFile20             = BrainVoyager.FindChild("leFile20");
    
    // connect widgets to functions
    startButton.clicked.connect(this, this.startProcessing); // 01-07-16
   // cancelButton.clicked.connect(this, this.reject); // arranged via QtCreator.
    getFileButton.clicked.connect(this, this.getFiles);
    btnLoadUFF.clicked.connect(this, this.getUFFFile);
    
}

returnScriptObj = function() { 
	return scriptObj; 
} 

returnScriptObj(); // should be after returnScriptOjb = function()
